/*
Stolyarevskiy Sergey, acobw.narod.ru

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation.
	
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
		
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

CASAB Version 1.1(2), 05.05.2007
*/

#include <windows.h>
#include <stdio.h>
#include <commctrl.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"
#include "resource.h"
#include "filter.h"

inline int R_G_B(int r, int g, int b){return (r<<16 | g<<8 | b);}
/*
inline int GetR(int rgb){return (rgb>>16) & 0xff;}
inline int GetG(int rgb){return (rgb>>8) & 0xff;}
inline int GetB(int rgb){return rgb & 0xff;}
*/
int RunProc(const FilterActivation *fa, const FilterFunctions *ff); 
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
int InitProc(FilterActivation *fa, const FilterFunctions *ff);

typedef struct MyFilterData {
	IFilterPreview		*ifp;
    unsigned char   *old_data;
	int NNoise;
	int Border;
	int NUP;
	int NLEFT;
	int NRIGHT;
	int NDOWN;
	int NNoise_old;
	int Border_old;
	int NUP_old;
	int NLEFT_old;
	int NRIGHT_old;
	int NDOWN_old;
	int HH;
	int WW;
} MyFilterData;

ScriptFunctionDef _func_defs[]={
    { (ScriptFunctionPtr)ScriptConfig, "Config", "0iiiii" },
    { NULL },
};

CScriptObject _obj={
    NULL, _func_defs
};

struct FilterDefinition filterDef_ = {
    NULL, NULL, NULL,									// next, prev, module
    "Centring and add border, version 1.1(2) ",            // name
    "Centring a scene and border of addition",			// desc
    "Stolyarevskiy Sergey",								// maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size
    InitProc,               // initProc
    NULL,                   // deinitProc
    RunProc,				// runProc
	NULL,				    // paramProc
	ConfigProc,				// configProc
    StringProc,				// stringProc
	NULL,					// startProc
	NULL,					// endProc
    &_obj,					// script_obj
    FssProc,				// fssProc
};

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_ = ff->addFilter(fm, &filterDef_, sizeof(FilterDefinition))))
        return 1;

    vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
    vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_);
}

//-----------------------------------------------------------------------------------------------------------------
int RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
MyFilterData *mfd = (MyFilterData *)fa->filter_data;
  Pixel32 *src,*dst;
int	th=fa->src.h;
int	tw=fa->src.w;
int wwhh=th*tw;
	src = (Pixel32 *)fa->src.data;
    dst = (Pixel32 *)fa->dst.data;
int i=(fa->src.modulo)>>2;

int SY=(mfd->NDOWN-mfd->NUP)>>1;
int SX=(mfd->NLEFT-mfd->NRIGHT)>>1;

int SYB=(mfd->NDOWN+mfd->NUP)>>1;
int SXB=(mfd->NLEFT+mfd->NRIGHT)>>1;

int ys,xs;
//***************************************************************************************
if (!mfd->Border)
{
for (int yt = 0; yt < th; yt++){
	for (int xt = 0; xt < tw; xt++){
ys=yt+SY;
xs=xt+SX;
if (ys<0)ys=0;
if (ys>=th)ys=th-1;
if (xs<0) xs=0;
if (xs>=tw) xs=tw-1;
int P=src[xs+ys*(tw+i)];
if(mfd->NNoise){
/*
//       
	if(yt>th-SYB) P = R_G_B(0,0,0);
	if(yt<SYB-1) P = R_G_B(0,0,0);
	if(xt>tw-SXB) P = R_G_B(0,0,0);
	if(xt<SXB-1) P = R_G_B(0,0,0);
*/
	if(yt>th-SYB) P = 0;
	if(yt<SYB-1) P = 0;
	if(xt>tw-SXB) P = 0;
	if(xt<SXB-1) P = 0;
}
dst[xt+yt*(tw+i)]=P;
}
	}
//***************************************************************************************
}else{
//***************************************************************************************
//   
	for (int yt = 0; yt < th; yt++){
	for (int xt = 0; xt < tw; xt++){
			int P=src[xt+yt*(tw+i)];
			if(yt==mfd->NDOWN-1 && xt<=(fa->src.w-mfd->NRIGHT) && xt>=mfd->NLEFT-1) P = R_G_B(255,0,0);
			if(xt==(fa->src.w-mfd->NRIGHT) && yt>=mfd->NDOWN-1 && yt<=(fa->src.h-mfd->NUP)) P = R_G_B(255,0,0);
			if(xt==mfd->NLEFT-1 && yt>=mfd->NDOWN-1 && yt<=(fa->src.h-mfd->NUP)) P = R_G_B(255,0,0);
			if(yt==(fa->src.h-mfd->NUP) && xt<=(fa->src.w-mfd->NRIGHT) && xt>=mfd->NLEFT-1) P = R_G_B(255,0,0);
			*dst++=P;	
}
        dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
	}
}
//***********************************************************************************		
		return 0;
}

BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);
    switch(msg) {
        case WM_INITDIALOG:
			SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;

			SendDlgItemMessage (hdlg, IDC_SLIDER_UP, TBM_SETRANGE, false, MAKELONG (1, 30));
			SendDlgItemMessage (hdlg, IDC_SLIDER_UP, TBM_SETPOS, true, mfd->NUP);

			SendDlgItemMessage (hdlg, IDC_SLIDER_LEFT, TBM_SETRANGE, false, MAKELONG (1, 30));
			SendDlgItemMessage (hdlg, IDC_SLIDER_LEFT, TBM_SETPOS, true, mfd->NLEFT);

			SendDlgItemMessage (hdlg, IDC_SLIDER_RIGHT, TBM_SETRANGE, false, MAKELONG (1, 30));
			SendDlgItemMessage (hdlg, IDC_SLIDER_RIGHT, TBM_SETPOS, true, mfd->NRIGHT);

			SendDlgItemMessage (hdlg, IDC_SLIDER_DOWN, TBM_SETRANGE, false, MAKELONG (1, 30));
			SendDlgItemMessage (hdlg, IDC_SLIDER_DOWN, TBM_SETPOS, true, mfd->NDOWN);

			CheckDlgButton(hdlg, IDC_BORDER, mfd->Border?BST_CHECKED:BST_UNCHECKED);
			CheckDlgButton(hdlg, IDC_NOIS, mfd->NNoise?BST_CHECKED:BST_UNCHECKED);
			mfd->NUP_old=mfd->NUP;
			mfd->NLEFT_old=mfd->NLEFT;
			mfd->NRIGHT_old=mfd->NRIGHT;
			mfd->NDOWN_old=mfd->NDOWN;
			mfd->NNoise_old=mfd->NNoise;
			mfd->Border_old=mfd->Border;
      mfd->ifp->InitButton(GetDlgItem(hdlg, IDPREVIEW));
            return TRUE;

	case WM_HSCROLL:
			if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_UP))
			{
				mfd->NUP=SendDlgItemMessage (hdlg, IDC_SLIDER_UP, TBM_GETPOS, 0, 0);
				
				if (mfd->NUP>mfd->HH-mfd->NDOWN-10){
					mfd->NDOWN=mfd->HH-10-mfd->NUP;
					SendDlgItemMessage (hdlg, IDC_SLIDER_DOWN, TBM_SETPOS, true, mfd->NDOWN);
				}
				mfd->ifp->RedoFrame();
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_LEFT))
			{
				mfd->NLEFT=SendDlgItemMessage (hdlg, IDC_SLIDER_LEFT, TBM_GETPOS, 0, 0);
				
				if (mfd->NLEFT>mfd->WW-mfd->NRIGHT-10){
					mfd->NRIGHT=mfd->WW-10-mfd->NLEFT;
				SendDlgItemMessage (hdlg, IDC_SLIDER_RIGHT, TBM_SETPOS, true, mfd->NRIGHT);
				}
				mfd->ifp->RedoFrame();
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_RIGHT))
			{
				mfd->NRIGHT=SendDlgItemMessage (hdlg, IDC_SLIDER_RIGHT, TBM_GETPOS, 0, 0);

				if (mfd->NRIGHT>mfd->WW-mfd->NLEFT-10){
					mfd->NLEFT=mfd->WW-10-mfd->NRIGHT;
				SendDlgItemMessage (hdlg, IDC_SLIDER_LEFT, TBM_SETPOS, true, mfd->NLEFT);
				}
				mfd->ifp->RedoFrame();
			}
			else if ((HWND) lParam == GetDlgItem(hdlg, IDC_SLIDER_DOWN))
			{
				mfd->NDOWN=SendDlgItemMessage (hdlg, IDC_SLIDER_DOWN, TBM_GETPOS, 0, 0);

				if (mfd->NDOWN>mfd->HH-mfd->NUP-10){
					mfd->NUP=mfd->HH-10-mfd->NDOWN;
					SendDlgItemMessage (hdlg, IDC_SLIDER_UP, TBM_SETPOS, true, mfd->NUP);
				}
				mfd->ifp->RedoFrame();
			}
			
			break;
     case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
				EndDialog(hdlg, 0);
            return TRUE;
			case IDPREVIEW:
				mfd->ifp->Toggle(hdlg);
				mfd->ifp->RedoFrame();
            return TRUE;
			case IDC_BORDER:
				mfd->Border= !!IsDlgButtonChecked(hdlg, IDC_BORDER);
				mfd->ifp->RedoFrame();
            return TRUE;
			case IDC_NOIS:
				mfd->NNoise= !!IsDlgButtonChecked(hdlg, IDC_NOIS);
				mfd->ifp->RedoFrame();
            return TRUE;
            case IDCANCEL:
				mfd->NUP=mfd->NUP_old;
				mfd->NLEFT=mfd->NLEFT_old;
				mfd->NRIGHT=mfd->NRIGHT_old;
				mfd->NDOWN=mfd->NDOWN_old;
				mfd->NNoise=mfd->NNoise_old;
				mfd->Border=mfd->Border_old;
				EndDialog(hdlg, 1);
            return FALSE;
            }
            break;
    }
return FALSE;
}

int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;	
	mfd->ifp = fa->ifp;
	return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER), hwnd,
            ConfigDlgProc, (LPARAM)fa->filter_data);
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
 	 MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	 sprintf(str," (%d, %d, %d, %d, %d)",mfd->NUP,mfd->NLEFT,mfd->NRIGHT,mfd->NDOWN,mfd->NNoise);
	}

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	mfd->NUP=argv[0].asInt();
	mfd->NLEFT=argv[1].asInt();
	mfd->NRIGHT=argv[2].asInt();
	mfd->NDOWN=argv[3].asInt();
	mfd->NNoise=argv[4].asInt();
}

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
      MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	  _snprintf(buf,buflen,"Config(%d,%d,%d,%d,%d)",mfd->NUP,mfd->NLEFT,mfd->NRIGHT,mfd->NDOWN,mfd->NNoise);
    return true;
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	mfd->NUP=1;
	mfd->NLEFT=1;
	mfd->NRIGHT=1;
	mfd->NDOWN=1;
	mfd->NNoise=0;
	mfd->HH=fa->src.h;
	mfd->WW=fa->src.w;
	return 0;
}