// File: loadimg.cpp
// Auth: Brian Allen Vanderburg II
// Date: Monday, December 29, 2003
// Desc: Routine to load a image into a bitmap for virtualdub

// This image will be used to do the blending/layering operation

#include <windows.h>
#include "filter.h"

#include "loadimg.h"
#include "win_bmp.h"

static int LoadBmp(VBitmap *dst, char *file, int x, int y);

VBitmap *LoadImg(VBitmap *src, Pixel32 bgcolor, char *file, int x, int y)
	{
	// First we want to allocate memory for the bitmap object
	PixDim i_w, i_h;
	Pixel32 *p;

	
	VBitmap *tmp;
	// I currently do not know if virtualdub allocate the memory
	// for the vbitmap and its data all at once or in 2 steps.
	// Here i do it in two steps for now
	// Basically, we want a perpect copy of source to use
	// then we fill the copy with our image to use as the layer
	
	// Allocate space for the vbitmap object
	tmp = (VBitmap*)GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, sizeof(VBitmap));
	if(tmp == NULL)
		return NULL;

	// Allocate space for the image data
	tmp->data = (Pixel32*)GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT, src->size);
	if(tmp->data == NULL)
		{
		GlobalFree((HGLOBAL)tmp);
		return NULL;
		}

	// Copy over some information
	tmp->palette = NULL;
	tmp->depth = src->depth;
	tmp->w = src->w;
	tmp->h = src->h;
	tmp->pitch = src->pitch;
	tmp->modulo = src->modulo;
	tmp->size = src->size;
	tmp->offset = src->offset; // What is this?

	// Fill the layer with the background color
	p = tmp->data;

	i_h = tmp->h;
	while(i_h--)
		{
		
		i_w = tmp->w;
		while(i_w--)
			{
			*p = bgcolor;

			p++;
			}

		p = (Pixel32*)((char*)p + tmp->modulo);
		}

	// Load the image into the layer at the correct position
	if(LoadBmp(tmp, file, x, y) == 0)
		{
		FreeImg(tmp);
		return NULL;
		}

	// Return the layer
	return tmp;
	}

void FreeImg(VBitmap *tmp)
	{
	if(tmp == NULL)
		return;

	if(tmp->data)
		GlobalFree((HGLOBAL)(tmp->data));

	GlobalFree((HGLOBAL)tmp);
	}


// Load the .bmp file into the layer at the correct position
static int LoadBmp(VBitmap *dst, char *file, int x, int y)
	{
	// TODO: Implement this function.
	// It should load the image at the correct position
	// in the layer.  If it is too big, it will be clipped
	// at the right and top.

	// X is the left offset
	// Y is the bottom offset

	// If there is no file to load, return success
	if(file == NULL)
		return 1;
	if(file[0] == '\0')
		return 1;	
	

	BITMAPINFOHEADER *bih;
	Pixel32 *pix;
	unsigned char r, g, b;
	int d_x, d_y;
	int w, h;

	bih = LoadBitmapFile(file);
	if(bih == NULL)
		{
		MessageBox(NULL, "Unable to load image\n", "Alert", MB_OK);
		return 0;
		}

	pix = dst->data;

	d_y = 0;
	h = dst->h;
	while(h--)
		{

		d_x = 0;
		w = dst->w;
		while(w--)
			{
			// Try to get the color value
			if(GetPixelRGB(bih, d_x - x, d_y - y, &r, &g, &b))
				{
				// Change the pixel if we were in the image
				*pix = (r << 16) + (g << 8) + b;
				}

			d_x++;
			pix++;
			}

		d_y++;
		pix = (Pixel32*)((unsigned char*)(pix) + dst->modulo);
		}


	FreeBitmap(bih);
	return 1;
	}

// Load a control image
int LoadCtrl(unsigned char *ctrl, char *file, int w, int h, int x, int y, int def)
	{
	// Fill the control array with 'ON' by default
	FillMemory(ctrl, w * h, def);

	// No control file, return success
	if(file == NULL)
		return 1;
	if(file[0] == '\0')
		return 1;

	BITMAPINFOHEADER *bih;
	int dx, dy;
	unsigned char gray;

	bih = LoadBitmapFile(file);
	if(bih == NULL)
		return 0;

	for(dy = 0; dy < h; dy++)
		{
		for(dx = 0; dx < w; dx++)
			{
			if(GetPixelGray(bih, (dx - x), (dy - y), &gray))
				{
				// If we are in the bounds of the image
				// Turn the value on or off
				// Note: If we are not in the bounds, it is
				// Untouched so the default will remain
				if(gray < 128)
					*ctrl = 0;
				else
					*ctrl = 1;
				}

			ctrl++;
			}
		}

	FreeBitmap(bih);

	return 1;
	}