/*
	File: vdf_chanmix.cpp
	Auth: Brian Allen Vanderburg II
	Date: Friday, November 21, 2003
	Desc: A VirtualDub Channel Mixer filter
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "filter.h"
#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"
#include "resource.h"

/* Prototypes */
extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff); 


/* Macro to get a color from the lookup table */
#define GETCOLOR(table, value, percent) (table[value * 201 + percent])

/* Data structure */
typedef struct tag_chanmixData
	{
	int rr, rg, rb; /* Red from Red, Green, Blue */
	int gr, gg, gb; /* Green from Red, Green Blue */
	int br, bg, bb; /* Blue from Red, Green, Blue */
	IFilterPreview *ifp;
	} chanmixData;

/* Globals */
static FilterDefinition *fd_chanmix;

static unsigned char *table = NULL;


/* Dialog procedure */
BOOL CALLBACK chanmixDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
	{
	/* Get filter activation, set during WM_INITDIALOG */
	chanmixData *data = (chanmixData*)GetWindowLong(hDlg, DWL_USER);
	char buf[1024];
	int pos;
	unsigned int ctrl;

	switch(uMsg)
		{
		case WM_INITDIALOG:
			/* Set user data to point to filter activation */
			SetWindowLong(hDlg, DWL_USER, lParam);
			data = (chanmixData*)(lParam);

			/* Set default range for sliders */
			SendDlgItemMessage(hDlg, IDC_RR, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_RG, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_RB, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_GR, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_GG, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_GB, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_BR, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_BG, TBM_SETRANGE, TRUE, MAKELONG(0, 200));
			SendDlgItemMessage(hDlg, IDC_BB, TBM_SETRANGE, TRUE, MAKELONG(0, 200));

			/* Set positions */
			SendDlgItemMessage(hDlg, IDC_RR, TBM_SETPOS, TRUE, data->rr);
			SendDlgItemMessage(hDlg, IDC_RG, TBM_SETPOS, TRUE, data->rg);
			SendDlgItemMessage(hDlg, IDC_RB, TBM_SETPOS, TRUE, data->rb);
			SendDlgItemMessage(hDlg, IDC_GR, TBM_SETPOS, TRUE, data->gr);
			SendDlgItemMessage(hDlg, IDC_GG, TBM_SETPOS, TRUE, data->gg);
			SendDlgItemMessage(hDlg, IDC_GB, TBM_SETPOS, TRUE, data->gb);
			SendDlgItemMessage(hDlg, IDC_BR, TBM_SETPOS, TRUE, data->br);
			SendDlgItemMessage(hDlg, IDC_BG, TBM_SETPOS, TRUE, data->bg);
			SendDlgItemMessage(hDlg, IDC_BB, TBM_SETPOS, TRUE, data->bb);

			wsprintf(buf, "%d", data->rr);
			SendDlgItemMessage(hDlg, IDC_RRT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->rg);
			SendDlgItemMessage(hDlg, IDC_RGT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->rb);
			SendDlgItemMessage(hDlg, IDC_RBT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->gr);
			SendDlgItemMessage(hDlg, IDC_GRT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->gg);
			SendDlgItemMessage(hDlg, IDC_GGT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->gb);
			SendDlgItemMessage(hDlg, IDC_GBT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->br);
			SendDlgItemMessage(hDlg, IDC_BRT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->bg);
			SendDlgItemMessage(hDlg, IDC_BGT, WM_SETTEXT, 0, (LPARAM)buf);
			wsprintf(buf, "%d", data->bb);
			SendDlgItemMessage(hDlg, IDC_BBT, WM_SETTEXT, 0, (LPARAM)buf);

			data->ifp->InitButton(GetDlgItem(hDlg, IDC_PREVIEW));

			break;

		case WM_HSCROLL:
			/* Update the text labels */
			pos = SendDlgItemMessage(hDlg, IDC_RR, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_RRT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_RG, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_RGT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_RB, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_RBT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_GR, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_GRT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_GG, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_GGT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_GB, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_GBT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_BR, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_BRT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_BG, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_BGT, WM_SETTEXT, 0, (LPARAM)buf);
			pos = SendDlgItemMessage(hDlg, IDC_BB, TBM_GETPOS, 0, 0);
			wsprintf(buf, "%d", pos);
			SendDlgItemMessage(hDlg, IDC_BBT, WM_SETTEXT, 0, (LPARAM)buf);

			break;

		case WM_COMMAND:
			ctrl = LOWORD(wParam);

			switch(ctrl)
				{
				case IDOK:
				case IDC_APPLY:
					/* End dialog, update changes */

					data->rr = SendDlgItemMessage(hDlg, IDC_RR, TBM_GETPOS, 0, 0);
					data->rg = SendDlgItemMessage(hDlg, IDC_RG, TBM_GETPOS, 0, 0);
					data->rb = SendDlgItemMessage(hDlg, IDC_RB, TBM_GETPOS, 0, 0);
					data->gr = SendDlgItemMessage(hDlg, IDC_GR, TBM_GETPOS, 0, 0);
					data->gg = SendDlgItemMessage(hDlg, IDC_GG, TBM_GETPOS, 0, 0);
					data->gb = SendDlgItemMessage(hDlg, IDC_GB, TBM_GETPOS, 0, 0);
					data->br = SendDlgItemMessage(hDlg, IDC_BR, TBM_GETPOS, 0, 0);
					data->bg = SendDlgItemMessage(hDlg, IDC_BG, TBM_GETPOS, 0, 0);
					data->bb = SendDlgItemMessage(hDlg, IDC_BB, TBM_GETPOS, 0, 0);

					if(ctrl == IDOK)
						EndDialog(hDlg, 0);

					if(ctrl == IDC_APPLY)
						data->ifp->RedoFrame();

					break;

				case IDC_PREVIEW:
					data->ifp->Toggle(hDlg);
					break;

				case IDCANCEL:
					/* End dialog, do not update changes */
					EndDialog(hDlg, 1);
					break;
				}
			break;

		case WM_CLOSE:
			/* This is the same as a cancel */
			EndDialog(hDlg, 1);
			break;

		default:
			/* Message was not handled */
			return FALSE;
		}
	/* Message was handled */
	return TRUE;
	}


/* Init procedure */
int _cdecl chanmixInitProc(FilterActivation *fa, const FilterFunctions *ff)
	{
	chanmixData *data = (chanmixData*)(fa->filter_data);

	/* Set default channel values */
	data->rr = 100; data->rg = 0; data->rb = 0;
	data->gr = 0; data->gg = 100; data->gb = 0;
	data->br = 0; data->bg = 0; data->bb = 100;

	return 0;
	}


/* Start procedure */
int __cdecl chanmixStartProc(FilterActivation *fa, const FilterFunctions *ff)
	{
	unsigned int color, percent;
	unsigned int tmp;

	/* Only build the table once */
	if(table)
		return 0;

	/* Build the lookup table */
	/* use GETCOLOR to get a translated color value */
	table = (unsigned char*)GlobalAlloc(GMEM_FIXED,
		256 * 201 * sizeof(unsigned char));

	if(table == NULL)
		return 1;
	
	for(color = 0; color < 256; color++)
		{
		for(percent = 0; percent <= 200; percent++)
			{
			tmp = (color * percent) / 100;
			if(tmp > 255)
				tmp = 255;

			table[color * 201 + percent] = tmp;
			}
		}


	return 0;
	}

/* End procedure */
int __cdecl chanmixEndProc(FilterActivation *fa, const FilterFunctions *ff)
	{
	if(table)
		GlobalFree((HGLOBAL)(table));

	table = NULL;

	return 0;
	}

/* Config procedure */
int __cdecl chanmixConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd)
	{
	((chanmixData*)(fa->filter_data))->ifp = fa->ifp;

	return DialogBoxParam(fa->filter->module->hInstModule,
		MAKEINTRESOURCE(IDD_CHANMIXDIALOG), hwnd,
		chanmixDlgProc, (LPARAM)(fa->filter_data));
	}

/* String procedure */
void __cdecl chanmixStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *buf)
	{
	chanmixData *data = (chanmixData*)(fa->filter_data);

	wsprintf(buf, " R(%d,%d,%d),G(%d,%d,%d),B(%d,%d,%d)",
		data->rr, data->rg, data->rb, data->gr, data->gg, data->gb, data->br, data->bg, data->bb);
	}

/* Run procedure */
int __cdecl chanmixRunProc(const FilterActivation *fa, const FilterFunctions *ff)
	{
	PixDim w, h;
	Pixel32 *src, *dst;
	unsigned int r, g, b;
	unsigned int nr, ng, nb;
	chanmixData *data;

	/* Get the data */
	data = (chanmixData*)(fa->filter_data);

	src = (Pixel32*)(fa->src.data);
	dst = (Pixel32*)(fa->dst.data);

	h = fa->src.h;

	/* Loop through the rows */
	while(h--)
		{
		w = fa->src.w;

		/* Loop through the columns */
		while(w--)
			{
			r = ((*src & 0x00FF0000) >> 16);
			g = ((*src & 0x0000FF00) >> 8);
			b = (*src & 0x000000FF);

			nr = GETCOLOR(table, r, data->rr) +
				GETCOLOR(table, g, data->rg) +
				GETCOLOR(table, b, data->rb);
			if(nr > 255)
				nr = 255;

			ng = GETCOLOR(table, r, data->gr) +
				GETCOLOR(table, g, data->gg) +
				GETCOLOR(table, b, data->gb);
			if(ng > 255)
				ng = 255;

			nb = GETCOLOR(table, r, data->br) +
				GETCOLOR(table, g, data->bg) +
				GETCOLOR(table, b, data->bb);
			if(nb > 255)
				nb = 255;

			*dst = (nr << 16) + (ng << 8) + nb;

			++src;
			++dst;
			}

		src = (Pixel32*)((char*)src + fa->src.modulo);
		dst = (Pixel32*)((char*)dst + fa->dst.modulo);

		}

	return 0;
	}


/* Create scripting support */

/* Config function */
void chanmixConfigFunc(IScriptInterpreter *isi, void *fa_ptr, CScriptValue *argv, int argc)
	{
	FilterActivation *fa = (FilterActivation*)(fa_ptr);
	chanmixData *data = (chanmixData*)(fa->filter_data);

	/* argc only has one possible value, 9 */

	data->rr = argv[0].asInt();
	if(data->rr < 0)
		data->rr = 0;
	if(data->rr > 200)
		data->rr = 200;

	data->rg = argv[1].asInt();
	if(data->rg < 0)
		data->rg = 0;
	if(data->rg > 200)
		data->rg = 200;

	data->rb = argv[2].asInt();
	if(data->rb < 0)
		data->rb = 0;
	if(data->rb > 200)
		data->rb = 200;

	data->gr = argv[3].asInt();
	if(data->gr < 0)
		data->gr = 0;
	if(data->gr > 200)
		data->gr = 200;

	data->gg = argv[4].asInt();
	if(data->gg < 0)
		data->gg = 0;
	if(data->gg > 200)
		data->gg = 200;

	data->gb = argv[5].asInt();
	if(data->gb < 0)
		data->gb = 0;
	if(data->gb > 200)
		data->gb = 200;
	
	data->br = argv[6].asInt();
	if(data->br < 0)
		data->br = 0;
	if(data->br > 200)
		data->br = 200;

	data->bg = argv[7].asInt();
	if(data->bg < 0)
		data->bg = 0;
	if(data->bg > 200)
		data->bg = 200;	

	data->bb = argv[8].asInt();
	if(data->bb < 0)
		data->bb = 0;
	if(data->bb > 200)
		data->bb = 200;

	}

/* Function defs */
ScriptFunctionDef chanmixFuncDefs[] =
	{
		{ (ScriptFunctionPtr)chanmixConfigFunc, "Config", "0iiiiiiiii" },
		{ NULL }
	};

/* Script object */
CScriptObject chanmixObject = 
	{
	NULL, chanmixFuncDefs, NULL
	};

/* Script output */
bool chanmixFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
	{
	chanmixData *data = (chanmixData*)(fa->filter_data);

	_snprintf(buf, buflen, "Config(%d,%d,%d,%d,%d,%d,%d,%d,%d)",
		data->rr, data->rg, data->rb,
		data->gr, data->gg, data->gb,
		data->br, data->bg, data->bb);

	return true;
	}


/* This is the filter definition */
struct FilterDefinition fdDef_chanmix =
	{
	NULL, NULL, NULL, /* next, prev, module */
	"Channel Mixer", /* name */
	"Mixes the color channels.", /* desc */
	"Brian Allen Vanderburg II", /* maker */
	NULL, /* private_data */
	sizeof(chanmixData), /* inst_data_size */
	chanmixInitProc, /* initProc */
	NULL, /* deinitProc */
	chanmixRunProc, /* runProc */
	NULL, /* paramProc */
	chanmixConfigProc, /* configProc */
	chanmixStringProc, /* stringProc */
	chanmixStartProc, /* startProc */
	chanmixEndProc, /* endProc */
	&chanmixObject, /* script_obj */
	chanmixFssProc /*fssProc */
	};

/* init function */
int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat)
	{
	InitCommonControls();

	fd_chanmix = ff->addFilter(fm, &fdDef_chanmix, sizeof(FilterDefinition));
	if(!fd_chanmix)
		return 1;

	vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;


	return 0;
	}

/* deinit function */
void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff)
	{
	ff->removeFilter(fd_chanmix);
	}

/* Dll Main */
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpReserved)
	{
	return TRUE;
	}
